/*
 * Written by Ed Hong UWT Feb. 17, 2003.
 * Modified by Donald Chinn May 14, 2003.
 * Then Modified by Ed Hong May 19, 2004.
 */

import java.util.*;

/**
 * A class that represents a graph.
 *
 * @author edhong
 * @version 0.0
 */

class edgeIterator implements Iterator{
	Edge [][] M; // reference to variable that stores edges
	int vindex; // index of vertex that we are iterating over
	int currpos;   // current position we are at
	int numvert; // total number of vertices

	// invariant for currpos: currpos is either the index of the
	//   next Edge to be returned, or currpos > nvert,
	//   indicating there are no more items.
	// this invariant holds after any call to the methods
	//   below.

	// construct a new iterator over vertex vnum on mat.
	public edgeIterator(Edge[][] mat, int vind, int nvert) {
		M = mat;
		vindex = vind;
		numvert = nvert;
		currpos = 0;
		// find the next item.
		while ( (currpos < nvert) && (M[vindex][currpos] == null))
			currpos++;
	}

	public boolean hasNext() {
		// with the invariant on currpos,
		// either currpos > nvert, or currpos is the index
		// of a valid Edge object.
		// no need to go searching for an item!
		if (currpos >= numvert)
			return false;
		return true;
	}

	public Object next() {
		Edge retval;
		// with the invariant on currpos,
		// either currpos > nvert, or currpos is the index
		// of a valid Edge object.
		// no need to go searching for an item!

		if (currpos >= numvert) {
			throw new NoSuchElementException();
		}
		retval = M[vindex][currpos];// M[vnum][currpos] must be a valid Edge.

		currpos++; // advance pointer to look for next item.

		// find the next item (keeps invariant on currpos true).
		while ( (currpos < numvert) && (M[vindex][currpos] == null))
			currpos++;
		return retval;
	}

	public void remove() {
		throw new UnsupportedOperationException();
	}

}

public class SimpleGraph {

    LinkedList vertexList;
    LinkedList edgeList;
    Edge [][] mat;


    // Constructors
    public SimpleGraph(int maxnumvertices) {
        this.vertexList = new LinkedList();
        this.edgeList = new LinkedList();
        mat = new Edge[maxnumvertices][maxnumvertices];
    }
    public SimpleGraph() {
		this(250); // Call SimpleGraph(250) constructor.
    }

    /**
     * Return the vertex list of this graph.
     * @returns  vertex list of this graph
     */
    public Iterator vertices() {
        return vertexList.iterator();
    }

    /**
     * Return the edge list of this graph.
     * @returns edge list of this graph
     */
    public Iterator edges() {
        return edgeList.iterator();
    }

    /**
     * Given a vertex, return an iterator to the edge list of that vertex
     * @param v  a vertex
     * @returns  an iterator to the edge list of that vertex
     */
    public Iterator incidentEdges(Vertex v) {
		// return a new iterator that marches through
		// the edge data
		return new edgeIterator(mat, v.getKey(),
								this.numVertices());

		// alternative implementation: add edges you find into
		// a linkedList, and return an iterator to the new
		// linked list.
/*		int i;
		LinkedList lst = new LinkedList();
		for (i=0; i < this.numVertices(); i++) {
			if (mat[v.getKey()][i] != null)
				lst.add(mat[v.getKey()][i]);
		}
		return lst.iterator();
*/
	}

    /**
     * Return an arbitrary vertex of this graph
     * @returns  some vertex of this graph
     */
    public Vertex aVertex() {
        if (vertexList.size() > 0)
            return (Vertex) vertexList.getFirst();
        else
            return null;
    }

    /**
     * Add a vertex to this graph.
     * @param o  an object to be associated with the new vertex
     * @returns  the new vertex
     */
    public Vertex insertVertex(Object o) {
        Vertex v;
        // create new vertex.
        v = new Vertex(o, vertexList.size());
        vertexList.addLast(v);
        return v;
    }

    /**
     * Add an edge to this graph.
     * @param v  the first endpoint of the edge
     * @param w  the second endpoint of the edge
     * @param o  an object to be associated with the new edge
     * @returns  the new edge
     */
    public Edge insertEdge(Vertex v, Vertex w, Object o) {
        Edge e;
        e = new Edge(v, w, o);
        edgeList.addLast(e);
        mat[v.getKey()][w.getKey()] = e;
        mat[w.getKey()][v.getKey()] = e;
        return e;
    }

    /**
     * Given a vertex and an edge, if the vertex is one of the endpoints
     * of the edge, return the other endpoint of the edge.  Otherwise,
     * return null.
     * @param v  a vertex
     * @param e  an edge
     * @returns  the other endpoint of the edge (or null, if v is not an endpoint of e)
     */
    public Vertex opposite(Vertex v, Edge e) {
        Vertex w;

        if (e.v1 == v) {
            w= e.v2;
        }
        else if (e.v2 == v) {
            w = e.v1;
        }
        else
            w = null;

        return w;
    }

    /**
     * Return the number of vertices in this graph.
     * @returns  the number of vertices
     */
    public int numVertices() {
        return vertexList.size();
    }

    /**
     * Return the number of edges in this graph.
     * @returns  the number of edges
     */
    public int numEdges() {
        return edgeList.size();
    }

    /**
     * Code to test the correctness of the SimpleGraph methods.
     */
    public static void main(String[] args) {
        // create graph a----b-----c,
        //                X     Y
        // X and Y are objects stored at edges. .

        // All Objects stored will be strings.

        SimpleGraph G = new SimpleGraph();
        Vertex v, w, a, b, c;
        Edge e, x, y;
        v = G.insertVertex("a");
        a = v;
        w = G.insertVertex("b");
        b = w;
        e = G.insertEdge(v,w,"X");
        x = e;
        v = G.insertVertex("c");
        c = v;
        e = G.insertEdge(w,v,"Y");
        y = e;

        Iterator i;

        System.out.println("Iterating through vertices...");
        for (i= G.vertices(); i.hasNext(); ) {
            v = (Vertex) i.next();
            System.out.println("found vertex " + v.element());
        }

        System.out.println("Iterating through adjacency lists...");
        for (i= G.vertices(); i.hasNext(); ) {
            v = (Vertex) i.next();
            System.out.println("Vertex "+v.element());
            Iterator j;

            for (j = G.incidentEdges(v); j.hasNext();) {
                e = (Edge) j.next();
                System.out.println("  found edge " + e.element());
            }
        }

        System.out.println("Testing opposite...");
        System.out.println("aXbYc is ");
        System.out.println(a);
        System.out.println(x);
        System.out.println(b);
        System.out.println(y);
        System.out.println(c);

        System.out.println("opposite(a,x) is " + G.opposite(a,x));
        System.out.println("opposite(a,y) is " + G.opposite(a,y));
        System.out.println("opposite(b,x) is " + G.opposite(b,x));
        System.out.println("opposite(b,y) is " + G.opposite(b,y));
        System.out.println("opposite(c,x) is " + G.opposite(c,x));
        System.out.println("opposite(c,y) is " + G.opposite(c,y));

    }
}




